package main

import (
	"fmt"
	"git.kuschku.de/justjanne/stateless-matrix-bot/api"
	"io"
	"log"
	"net/http"
	"strings"
	"time"
)

type MatrixBot struct {
	token    *api.Token
	handlers map[string]func(bot *MatrixBot, notification api.Notification) error
}

func NewMatrixBot() *MatrixBot {
	return &MatrixBot{
		token:    nil,
		handlers: make(map[string]func(bot *MatrixBot, notification api.Notification) error),
	}
}

func (bot *MatrixBot) RefreshToken() error {
	if bot.token == nil {
		return fmt.Errorf("no refresh token available")
	}
	userData, err := api.Refresh(bot.token.RefreshToken)
	if err != nil {
		return err
	}
	bot.token = &api.Token{
		AccessToken:  userData.AccessToken,
		Expires:      time.Now().Add(time.Duration(userData.ExpiresInMs) / 2 * time.Millisecond),
		RefreshToken: userData.RefreshToken,
	}
	return nil
}

func (bot *MatrixBot) Login(username string, password string, deviceId string) error {
	userData, err := api.Login(username, password, deviceId)
	if err != nil {
		return err
	}
	bot.token = &api.Token{
		AccessToken:  userData.AccessToken,
		Expires:      time.Now().Add(time.Duration(userData.ExpiresInMs) / 2 * time.Millisecond),
		RefreshToken: userData.RefreshToken,
	}
	return nil
}

func (bot *MatrixBot) RefreshTask() {
	for true {
		if bot.token != nil && time.Now().After(bot.token.Expires) {
			if err := bot.RefreshToken(); err != nil {
				log.Printf("error refresh token: %s\n", err.Error())
			}
		}
		time.Sleep(1 * time.Second)
	}
}

func (bot *MatrixBot) RegisterPusher(url string) error {
	return api.SetPusher(*bot.token, url)
}

func (bot *MatrixBot) HandleFunc(command string, handler func(bot *MatrixBot, notification api.Notification) error) {
	bot.handlers[command] = handler
}

func (bot *MatrixBot) Serve(endpoint string) {
	http.HandleFunc("/healthz", func(writer http.ResponseWriter, request *http.Request) {
		writer.WriteHeader(200)
		_, _ = io.WriteString(writer, "OK\n")
	})
	http.HandleFunc("/_matrix/push/v1/notify", func(writer http.ResponseWriter, request *http.Request) {
		log.Println("Received push notification")
		notification, err := api.ParseNotification(request.Body)
		if err != nil {
			log.Println(err.Error())
			return
		}
		if notification.EventId == "" {
			return
		}
		err = api.SetReadReceipt(*bot.token, notification.RoomId, notification.EventId)
		if err != nil {
			log.Println(err.Error())
			return
		}
		command := strings.SplitN(notification.Content.Body, " ", 2)[0]
		handler, ok := bot.handlers[command]
		if !ok {
			log.Printf("could not find handler for '%s'\n", command)
			return
		}
		err = handler(bot, notification)
		if err != nil {
			log.Println(err.Error())
			return
		}
		writer.WriteHeader(204)
	})
	log.Fatal(http.ListenAndServe(endpoint, nil))
}
